"""
Choice ENUMs for easier maintenance of CharField's choice parameters. 

"""


from django_mri.models.choices.scanning_sequence import ScanningSequence
from django_mri.models.choices.sequence_variant import SequenceVariant
