"""
An app to manage MRI data. A Scan instance may be related to multiple file
formats and is meant to provide a unified model for dealing with the 
underlying data and header information.

"""


default_app_config = "django_mri.apps.DjangoMriConfig"
